﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Diagnostics;
using System.IO;
using MontoyaTech.Process.Net;

namespace MontoyaTech.ExtraConsole.Net
{
    /// <summary>
    /// The outline of a extra console that can be used to display information and status.
    /// </summary>
    public class ExtraConsole
    {
        private System.Diagnostics.Process _process = null;

        private StreamWriter _writer = null;
        
        /// <summary>
        /// Opens this Console so that it can be written to.
        /// </summary>
        public void Open()
        {
            if (this._process == null)
            {
                var startInfo = new ProcessStartInfo()
                {
                    FileName = Path.Combine(Environment.CurrentDirectory, "ExtraConsole.Net.exe"),
                    RedirectStandardInput = true
                };

                this._process = WinProcess.Create(startInfo, creationFlags: WinProcess.CreationFlags.CREATE_NEW_CONSOLE);

                this._writer = this._process.StandardInput;
            }
        }

        /// <summary>
        /// Writes a string and a new line to this Console if open.
        /// </summary>
        /// <param name="str"></param>
        /// <exception cref="InvalidOperationException"></exception>
        public void WriteLine(string str)
        {
            if (this._process == null || this._writer == null)
                throw new InvalidOperationException("Console must be open.");

            this._writer.WriteLine(str);
        }

        /// <summary>
        /// Writes a string to this Console if open.
        /// </summary>
        /// <param name="str"></param>
        /// <exception cref="InvalidOperationException"></exception>
        public void Write(string str)
        {
            if (this._process == null || this._writer == null)
                throw new InvalidOperationException("Console must be open.");

            this._writer.Write(str);
        }

        /// <summary>
        /// Closes this Console if open.
        /// </summary>
        public void Close()
        {
            try
            {
                this._process?.Kill();
            }
            finally
            {
                this._process = null;
            }

            try
            {
                this._writer?.Close();
            }
            finally
            {
                this._writer = null;
            }
        }

        ~ExtraConsole()
        {
            this.Close();
        }
    }
}
